<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Helper;

use \Monolog\Handler\StreamHandler;
use FiloBlu\MappConnector\Helper\ExceptionHelper;
use FiloBlu\MappConnector\Helper\LogHelper;

/**
 * Class FileLoggerHelper
 * @package FiloBlu\MappConnector\Helper
 */
class FileLoggerHelper
{
    /**
     * @var \Monolog\Logger
     */
    protected $_logger;

    /**
     * @var 
     */
    protected $_config;

    /**
     * @var array
     */
    protected $_cliColors = ['ERROR' => "\033[41m", 'WARNING' => "\033[43m\033[1;34m", 'DEBUG' => "\033[47m\033[1;34m", 'INFO' => "\033[44m"];

    /**
     * @var mixed
     */
    protected $_logLevel;

    /**
     * @var mixed
     */
    protected $_verboseCli;

    /**
     * FileLoggerHelper constructor.
     * @param \Magento\Framework\Filesystem $filesystem
     */
    public function __construct(
        \Magento\Framework\Filesystem $filesystem,
        \FiloBlu\MappConnector\Model\Config $config
    ) {
        $filename = 'mapp-connector.log';
        $channelname = 'mapp-connector';
        $this->_config = $config;
        $this->_logger = new \Monolog\Logger($channelname);
        $vardir = $filesystem
            ->getDirectoryRead(\Magento\Framework\App\Filesystem\DirectoryList::VAR_DIR)
            ->getAbsolutePath();

        $this->_logger->pushHandler(new StreamHandler(
            $vardir . 'log' . DIRECTORY_SEPARATOR . basename($filename)
        ));

        $this->_logLevel = $this->_config->getConfigValue('mappconnector/general/log_level');
        $this->_verboseCli = $this->_config->getConfigValue('mappconnector/general/enable_verbose_cli');
    }

    /**
     * @return \Monolog\Logger|\Monolog\Logger
     */
    public function getLogger()
    {
        return $this->_logger;
    }

    /**
     * @param $msg
     * @param array $context
     */
    public function info($msg,$context = [],$where = '')
    {
        $this->_logger->info($where. "\n".$msg, $context);
        LogHelper::setLog($msg);
        $this->verboseCli($msg,'INFO');
    }

    /**
     * @param $msg
     * @param array $context
     */
    public function debug($msg,$context = [],$where = '')
    {
        if(in_array($this->_logLevel, ['developer']))
        {
            $this->_logger->addDebug($where . "\n".$msg, $context);
        }
        $this->verboseCli($msg,'DEBUG');
    }

    /**
     * @param $msg
     * @param array $context
     */
    public function warning($msg,$context = [],$where = '')
    {
        if(in_array($this->_logLevel, ['developer','staging']))
        {
            $this->_logger->addWarning($where . "\n".$msg, $context);
        }
        $this->verboseCli($msg,'WARNING');
    }

    /**
     * @param $msg
     * @param array $context
     * @param string $where
     * @throws \Exception
     */
    public function error($msg,$context = [],$where = '')
    {
        if(in_array($this->_logLevel, ['developer','staging','production']))
        {
            $this->_logger->addError($where . "\n".$msg, $context);
        }
        ExceptionHelper::throwException($msg);
        $this->verboseCli($where.$msg,'ERROR');
    }

    /**
     * @param $msg
     * @param $from
     */
    public function verboseCli($msg,$from){
        if($this->_verboseCli && php_sapi_name() === 'cli'){
            echo $this->_cliColors[$from].$from."\033[0m: ".$msg."\n";
        }
    }

}